From: npiggin@suse.de
To: Andrew Morton <akpm@linux-foundation.org>
Cc: linux-fsdevel@vger.kernel.org,
Mark Fasheh <mark.fasheh@oracle.com>,
Jeff Dike <jdike@addtoit.com>
Subject: [patch 35/41] hostfs convert to new aops.
Date: Fri, 25 May 2007 22:22:19 +1000 [thread overview]
Message-ID: <20070524053158.611049000@linux.local0.net> (raw)
In-Reply-To: 20070524052844.860329000@suse.de
[-- Attachment #1: fs-hostfs-aops.patch --]
[-- Type: text/plain, Size: 3317 bytes --]
This also gets rid of a lot of useless read_file stuff. And also
optimises the full page write case by marking a !uptodate page uptodate.
Cc: Jeff Dike <jdike@addtoit.com>
Cc: Linux Filesystems <linux-fsdevel@vger.kernel.org>
Signed-off-by: Nick Piggin <npiggin@suse.de>
fs/hostfs/hostfs_kern.c | 70 +++++++++++++++++++-----------------------------
1 file changed, 28 insertions(+), 42 deletions(-)
Index: linux-2.6/fs/hostfs/hostfs_kern.c
===================================================================
--- linux-2.6.orig/fs/hostfs/hostfs_kern.c
+++ linux-2.6/fs/hostfs/hostfs_kern.c
@@ -466,56 +466,42 @@ int hostfs_readpage(struct file *file, s
return err;
}
-int hostfs_prepare_write(struct file *file, struct page *page,
- unsigned int from, unsigned int to)
+int hostfs_write_begin(struct file *file, struct address_space *mapping,
+ loff_t pos, unsigned len, unsigned flags,
+ struct page **pagep, void **fsdata)
{
- char *buffer;
- long long start, tmp;
- int err;
+ pgoff_t index = pos >> PAGE_CACHE_SHIFT;
- start = (long long) page->index << PAGE_CACHE_SHIFT;
- buffer = kmap(page);
- if(from != 0){
- tmp = start;
- err = read_file(FILE_HOSTFS_I(file)->fd, &tmp, buffer,
- from);
- if(err < 0) goto out;
- }
- if(to != PAGE_CACHE_SIZE){
- start += to;
- err = read_file(FILE_HOSTFS_I(file)->fd, &start, buffer + to,
- PAGE_CACHE_SIZE - to);
- if(err < 0) goto out;
- }
- err = 0;
- out:
- kunmap(page);
- return err;
+ *pagep = __grab_cache_page(mapping, index);
+ if (!*pagep)
+ return -ENOMEM;
+ return 0;
}
-int hostfs_commit_write(struct file *file, struct page *page, unsigned from,
- unsigned to)
+int hostfs_write_end(struct file *file, struct address_space *mapping,
+ loff_t pos, unsigned len, unsigned copied,
+ struct page *page, void *fsdata)
{
- struct address_space *mapping = page->mapping;
struct inode *inode = mapping->host;
- char *buffer;
- long long start;
- int err = 0;
+ void *buffer;
+ unsigned from = pos & (PAGE_CACHE_SIZE - 1);
+ int err;
- start = (((long long) page->index) << PAGE_CACHE_SHIFT) + from;
buffer = kmap(page);
- err = write_file(FILE_HOSTFS_I(file)->fd, &start, buffer + from,
- to - from);
- if(err > 0) err = 0;
-
- /* Actually, if !err, write_file has added to-from to start, so, despite
- * the appearance, we are comparing i_size against the _last_ written
- * location, as we should. */
+ err = write_file(FILE_HOSTFS_I(file)->fd, &pos, buffer + from, copied);
+ kunmap(page);
- if(!err && (start > inode->i_size))
- inode->i_size = start;
+ if (!PageUptodate(page) && err == PAGE_CACHE_SIZE)
+ SetPageUptodate(page);
+ unlock_page(page);
+ page_cache_release(page);
+
+ /* If err > 0, write_file has added err to pos, so we are comparing
+ * i_size against the last byte written.
+ */
+ if (err > 0 && (pos > inode->i_size))
+ inode->i_size = pos;
- kunmap(page);
return err;
}
@@ -523,8 +509,8 @@ static const struct address_space_operat
.writepage = hostfs_writepage,
.readpage = hostfs_readpage,
.set_page_dirty = __set_page_dirty_nobuffers,
- .prepare_write = hostfs_prepare_write,
- .commit_write = hostfs_commit_write
+ .write_begin = hostfs_write_begin,
+ .write_end = hostfs_write_end,
};
static int init_inode(struct inode *inode, struct dentry *dentry)
--
next prev parent reply other threads:[~2007-05-25 12:37 UTC|newest]
Thread overview: 77+ messages / expand[flat|nested] mbox.gz Atom feed top
2007-05-25 12:21 [patch 00/41] Buffered write deadlock fix and new aops for 2.6.22-rc2-mm1 npiggin
2007-05-25 12:21 ` [patch 01/41] mm: revert KERNEL_DS buffered write optimisation npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 02/41] Revert 81b0c8713385ce1b1b9058e916edcf9561ad76d6 npiggin
2007-05-25 12:21 ` npiggin, Andrew Morton
2007-05-25 12:21 ` [patch 03/41] Revert 6527c2bdf1f833cc18e8f42bd97973d583e4aa83 npiggin
2007-05-25 12:21 ` npiggin, Andrew Morton
2007-05-25 12:21 ` [patch 04/41] mm: clean up buffered write code npiggin
2007-05-25 12:21 ` npiggin, Andrew Morton
2007-05-25 12:21 ` [patch 05/41] mm: debug write deadlocks npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 06/41] mm: trim more holes npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 07/41] mm: buffered write cleanup npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 08/41] mm: write iovec cleanup npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 09/41] mm: fix pagecache write deadlocks npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 10/41] mm: buffered write iterator npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 11/41] fs: fix data-loss on error npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 12/41] fs: introduce write_begin, write_end, and perform_write aops npiggin
2007-05-25 12:21 ` npiggin
2007-05-31 4:30 ` Andrew Morton
2007-05-31 4:30 ` Andrew Morton
2007-05-31 4:43 ` Nick Piggin
2007-05-31 4:43 ` Nick Piggin
2007-05-31 4:52 ` Andrew Morton
2007-05-31 4:52 ` Andrew Morton
2007-05-31 4:57 ` Nick Piggin
2007-05-31 4:57 ` Nick Piggin
2007-05-31 5:11 ` Andrew Morton
2007-05-31 5:11 ` Andrew Morton
2007-05-31 5:15 ` Nick Piggin
2007-05-31 5:15 ` Nick Piggin
2007-05-31 7:05 ` Andrew Morton
2007-05-31 7:05 ` Andrew Morton
2007-06-01 1:18 ` Nick Piggin
2007-06-01 1:18 ` Nick Piggin
2007-05-25 12:21 ` [patch 13/41] mm: restore KERNEL_DS optimisations npiggin
2007-05-25 12:21 ` npiggin
2007-05-25 12:21 ` [patch 14/41] implement simple fs aops npiggin
2007-05-25 12:21 ` [patch 15/41] block_dev convert to new aops npiggin
2007-05-25 12:22 ` [patch 16/41] ext2 " npiggin
2007-05-25 12:22 ` [patch 17/41] ext3 " npiggin
2007-05-25 12:22 ` [patch 18/41] ext4 " npiggin
2007-05-25 12:22 ` [patch 19/41] xfs " npiggin
2007-05-25 12:22 ` [patch 20/41] fs: new cont helpers npiggin
2007-05-25 12:22 ` [patch 21/41] fat convert to new aops npiggin
2007-05-25 12:22 ` [patch 22/41] adfs " npiggin
2007-05-26 9:14 ` Russell King
2007-05-27 1:35 ` Nick Piggin
2007-05-25 12:22 ` [patch 23/41] hfs " npiggin
2007-05-25 12:22 ` [patch 24/41] hfsplus " npiggin
2007-05-25 12:22 ` [patch 25/41] hpfs " npiggin
2007-05-25 12:22 ` [patch 26/41] bfs " npiggin
2007-05-25 12:22 ` [patch 27/41] qnx4 " npiggin
2007-05-25 14:35 ` Anders Larsen
2007-05-26 7:23 ` Nick Piggin
2007-05-25 12:22 ` [patch 28/41] reiserfs use generic write npiggin
2007-05-25 12:22 ` [patch 29/41] reiserfs convert to new aops npiggin
2007-05-25 12:22 ` [patch 30/41] reiserfs use generic_cont_expand_simple npiggin
2007-05-25 12:22 ` [patch 31/41] With reiserfs no longer using the weird generic_cont_expand, remove it completely npiggin
2007-05-25 12:22 ` [patch 32/41] nfs convert to new aops npiggin
2007-05-25 12:22 ` [patch 33/41] smb " npiggin
2007-05-25 12:22 ` [patch 34/41] fuse " npiggin
2007-05-25 12:22 ` npiggin [this message]
2007-05-25 12:22 ` [patch 36/41] jffs2 " npiggin
2007-05-25 12:22 ` [patch 37/41] ufs " npiggin
2007-05-25 12:22 ` [patch 38/41] udf " npiggin
2007-05-25 12:22 ` [patch 39/41] sysv " npiggin
2007-05-25 12:22 ` [patch 40/41] minix " npiggin
2007-05-25 12:22 ` [patch 41/41] jfs " npiggin
2007-05-25 22:55 ` [patch 00/41] Buffered write deadlock fix and new aops for 2.6.22-rc2-mm1 Mark Fasheh
-- strict thread matches above, loose matches on Subject: below --
2007-05-14 6:06 [patch 00/41] Buffered write deadlock fix and new aops for 2.6.21-mm2 npiggin
2007-05-14 6:06 ` [patch 35/41] hostfs convert to new aops npiggin
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20070524053158.611049000@linux.local0.net \
--to=npiggin@suse.de \
--cc=akpm@linux-foundation.org \
--cc=jdike@addtoit.com \
--cc=linux-fsdevel@vger.kernel.org \
--cc=mark.fasheh@oracle.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.